@@ -1,9 +1,12 @@
+import os
+import requests
+from requests.adapters import HTTPAdapter
+
 import torch
 from torch import nn
 from torch.nn import functional as F
-import requests
-from requests.adapters import HTTPAdapter
-import os
+
+from .utils.download import download_url_to_file
 
 
 class BasicConv2d(nn.Module):
@@ -208,10 +211,8 @@ class InceptionResnetV1(nn.Module):
             tmp_classes = 8631
         elif pretrained == 'casia-webface':
             tmp_classes = 10575
-        elif pretrained is None and self.num_classes is None:
-            raise Exception('At least one of "pretrained" or "num_classes" must be specified')
-        else:
-            tmp_classes = self.num_classes
+        elif pretrained is None and self.classify and self.num_classes is None:
+            raise Exception('If "pretrained" is not specified and "classify" is True, "num_classes" must be specified')
 
 
         # Define layers
@@ -255,12 +256,12 @@ class InceptionResnetV1(nn.Module):
         self.dropout = nn.Dropout(dropout_prob)
         self.last_linear = nn.Linear(1792, 512, bias=False)
         self.last_bn = nn.BatchNorm1d(512, eps=0.001, momentum=0.1, affine=True)
-        self.logits = nn.Linear(512, tmp_classes)
 
         if pretrained is not None:
+            self.logits = nn.Linear(512, tmp_classes)
             load_weights(self, pretrained)
 
-        if self.num_classes is not None:
+        if self.classify and self.num_classes is not None:
             self.logits = nn.Linear(512, self.num_classes)
 
         self.device = torch.device('cpu')
@@ -269,13 +270,13 @@ class InceptionResnetV1(nn.Module):
             self.to(device)
 
     def forward(self, x):
-        """Calculate embeddings or probabilities given a batch of input image tensors.
+        """Calculate embeddings or logits given a batch of input image tensors.
 
         Arguments:
             x {torch.tensor} -- Batch of image tensors representing faces.
 
         Returns:
-            torch.tensor -- Batch of embeddings or softmax probabilities.
+            torch.tensor -- Batch of embedding vectors or multinomial logits.
         """
         x = self.conv2d_1a(x)
         x = self.conv2d_2a(x)
@@ -312,29 +313,20 @@ def load_weights(mdl, name):
         ValueError: If 'pretrained' not equal to 'vggface2' or 'casia-webface'.
     """
     if name == 'vggface2':
-        features_path = 'https://drive.google.com/uc?export=download&id=1cWLH_hPns8kSfMz9kKl9PsG5aNV2VSMn'
-        logits_path = 'https://drive.google.com/uc?export=download&id=1mAie3nzZeno9UIzFXvmVZrDG3kwML46X'
+        path = 'https://github.com/timesler/facenet-pytorch/releases/download/v2.2.9/20180402-114759-vggface2.pt'
     elif name == 'casia-webface':
-        features_path = 'https://drive.google.com/uc?export=download&id=1LSHHee_IQj5W3vjBcRyVaALv4py1XaGy'
-        logits_path = 'https://drive.google.com/uc?export=download&id=1QrhPgn1bGlDxAil2uc07ctunCQoDnCzT'
+        path = 'https://github.com/timesler/facenet-pytorch/releases/download/v2.2.9/20180408-102900-casia-webface.pt'
     else:
         raise ValueError('Pretrained models only exist for "vggface2" and "casia-webface"')
 
     model_dir = os.path.join(get_torch_home(), 'checkpoints')
     os.makedirs(model_dir, exist_ok=True)
 
-    state_dict = {}
-    for i, path in enumerate([features_path, logits_path]):
-        cached_file = os.path.join(model_dir, '{}_{}.pt'.format(name, path[-10:]))
-        if not os.path.exists(cached_file):
-            print('Downloading parameters ({}/2)'.format(i+1))
-            s = requests.Session()
-            s.mount('https://', HTTPAdapter(max_retries=10))
-            r = s.get(path, allow_redirects=True)
-            with open(cached_file, 'wb') as f:
-                f.write(r.content)
-        state_dict.update(torch.load(cached_file))
+    cached_file = os.path.join(model_dir, os.path.basename(path))
+    if not os.path.exists(cached_file):
+        download_url_to_file(path, cached_file)
 
+    state_dict = torch.load(cached_file)
     mdl.load_state_dict(state_dict)
 
 
