@@ -5,6 +5,7 @@ from torchvision.ops.boxes import batched_nms
 from PIL import Image
 import numpy as np
 import os
+import math
 
 # OpenCV is optional, but required if using numpy arrays instead of PIL
 try:
@@ -12,10 +13,23 @@ try:
 except:
     pass
 
+def fixed_batch_process(im_data, model):
+    batch_size = 512
+    out = []
+    for i in range(0, len(im_data), batch_size):
+        batch = im_data[i:(i+batch_size)]
+        out.append(model(batch))
+
+    return tuple(torch.cat(v, dim=0) for v in zip(*out))
 
 def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
     if isinstance(imgs, (np.ndarray, torch.Tensor)):
-        imgs = torch.as_tensor(imgs, device=device)
+        if isinstance(imgs,np.ndarray):
+            imgs = torch.as_tensor(imgs.copy(), device=device)
+
+        if isinstance(imgs,torch.Tensor):
+            imgs = torch.as_tensor(imgs, device=device)
+
         if len(imgs.shape) == 3:
             imgs = imgs.unsqueeze(0)
     else:
@@ -24,8 +38,9 @@ def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
         if any(img.size != imgs[0].size for img in imgs):
             raise Exception("MTCNN batch processing only compatible with equal-dimension images.")
         imgs = np.stack([np.uint8(img) for img in imgs])
+        imgs = torch.as_tensor(imgs.copy(), device=device)
 
-    imgs = torch.as_tensor(imgs, device=device)
+    
 
     model_dtype = next(pnet.parameters()).dtype
     imgs = imgs.permute(0, 3, 1, 2).type(model_dtype)
@@ -47,8 +62,11 @@ def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
     # First stage
     boxes = []
     image_inds = []
-    all_inds = []
+
+    scale_picks = []
+
     all_i = 0
+    offset = 0
     for scale in scales:
         im_data = imresample(imgs, (int(h * scale + 1), int(w * scale + 1)))
         im_data = (im_data - 127.5) * 0.0078125
@@ -57,17 +75,20 @@ def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
         boxes_scale, image_inds_scale = generateBoundingBox(reg, probs[:, 1], scale, threshold[0])
         boxes.append(boxes_scale)
         image_inds.append(image_inds_scale)
-        all_inds.append(all_i + image_inds_scale)
-        all_i += batch_size
+
+        pick = batched_nms(boxes_scale[:, :4], boxes_scale[:, 4], image_inds_scale, 0.5)
+        scale_picks.append(pick + offset)
+        offset += boxes_scale.shape[0]
 
     boxes = torch.cat(boxes, dim=0)
-    image_inds = torch.cat(image_inds, dim=0).cpu()
-    all_inds = torch.cat(all_inds, dim=0)
+    image_inds = torch.cat(image_inds, dim=0)
+
+    scale_picks = torch.cat(scale_picks, dim=0)
 
     # NMS within each scale + image
-    pick = batched_nms(boxes[:, :4], boxes[:, 4], all_inds, 0.5)
-    boxes, image_inds = boxes[pick], image_inds[pick]
-    
+    boxes, image_inds = boxes[scale_picks], image_inds[scale_picks]
+
+
     # NMS within each image
     pick = batched_nms(boxes[:, :4], boxes[:, 4], image_inds, 0.7)
     boxes, image_inds = boxes[pick], image_inds[pick]
@@ -91,7 +112,9 @@ def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
                 im_data.append(imresample(img_k, (24, 24)))
         im_data = torch.cat(im_data, dim=0)
         im_data = (im_data - 127.5) * 0.0078125
-        out = rnet(im_data)
+
+        # This is equivalent to out = rnet(im_data) to avoid GPU out of memory.
+        out = fixed_batch_process(im_data, rnet)
 
         out0 = out[0].permute(1, 0)
         out1 = out[1].permute(1, 0)
@@ -118,7 +141,9 @@ def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
                 im_data.append(imresample(img_k, (48, 48)))
         im_data = torch.cat(im_data, dim=0)
         im_data = (im_data - 127.5) * 0.0078125
-        out = onet(im_data)
+        
+        # This is equivalent to out = onet(im_data) to avoid GPU out of memory.
+        out = fixed_batch_process(im_data, onet)
 
         out0 = out[0].permute(1, 0)
         out1 = out[1].permute(1, 0)
@@ -146,6 +171,8 @@ def detect_face(imgs, minsize, pnet, rnet, onet, threshold, factor, device):
     boxes = boxes.cpu().numpy()
     points = points.cpu().numpy()
 
+    image_inds = image_inds.cpu()
+
     batch_boxes = []
     batch_points = []
     for b_i in range(batch_size):
